/* hand.cpp - Blackjack Neural Network
 * Copyright (C) 2007 Christopher Wellons <ccw129@psu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */ 

#include "hand.h"

hand::hand()
{
  recalc = true;
}

int hand::value()
{
  if (!recalc)
    return cur_val;
  
  int ace_count = 0;
  int val = 0;
  vector<int>::iterator ii;
  for (ii = held.begin(); ii != held.end(); ii++)
    {
      if (*ii == 1)
	{
	  // Ace
	  val+= 11;
	  ace_count++;
	}
      else
	{
	  val+= *ii;
	}
    }
  
  // Minimize ace values until we are under 21
  while (val > 21 && ace_count > 0)
    {
      val -= 10;
      ace_count--;
    }

  cur_val = val;
  recalc = false;
  return val;
}

int hand::min()
{
  int val = 0;
  vector<int>::iterator ii;
  for (ii = held.begin(); ii != held.end(); ii++)
    {
      val+= *ii;
    }
  
  return val;
}

void hand::take(int val)
{
  held.push_back(val);
  recalc = true;
}

void hand::fold(shoe &ret)
{
  ret.discard(held);
  recalc = true;
}

void hand::print()
{
  vector<int>::iterator ii;
  for (ii = held.begin(); ii != held.end(); ii++)
    {
      if (*ii == 1)
	cout << "A ";
      else
	cout << *ii << " ";
    }
  cout << endl;
}

void hand::display()
{
  cout << "(hole) ";
  
  vector<int>::iterator ii;
  for (ii = held.begin(), ii++; ii != held.end(); ii++)
    {
      if (*ii == 1)
	cout << "A ";
      else
	cout << *ii << " ";
    }
  cout << endl;
}
