/* hand.h - Blackjack Neural Network
 * Copyright (C) 2007 Christopher Wellons <ccw129@psu.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */ 

/* Simulates a blackjack hand. */

#ifndef HAND_H
#define HAND_H

#include <iostream>
#include <vector>
#include "shoe.h"
using namespace std;

// Hand class: a player's hand
class hand {
public:
  hand();
  
  // The value of the hand
  int value();

  // The minimum value the hand can currently be (ace = 1)
  int min();
  
  // Add card to hand
  void take(int);
  
  // Discard hand
  void fold(shoe&);
  
  // Print hand to cout
  void print();
  
  // Print without hole card
  void display();

private:
  // Current hand
  vector<int> held;

  // precalculated value
  bool recalc;
  int cur_val;
};

#endif
